      SUBROUTINE BCIMP( X, BCTYPE, BOTTOP, CPHS, CSHS, RHOHS,
     &   F, G, IPOW )

C     Compute Boundary Condition IMPedance

      INCLUDE 'common.f'

      REAL*8 YV( 5 )
      COMPLEX*16 CPHS, CSHS
      CHARACTER BCTYPE*1, BOTTOP*3

      IPOW = 0

C     *** Vacuum or Twersky ***

      IF ( BCTYPE(1:1) .EQ. 'V' .OR. 
     &     BCTYPE(1:1) .EQ. 'S' .OR. BCTYPE(1:1) .EQ. 'H' .OR.
     &     BCTYPE(1:1) .EQ. 'T' .OR. BCTYPE(1:1) .EQ. 'I' ) THEN
         F = 1.0
         G = 0.0
      ENDIF

C     *** Rigid ***

      IF ( BCTYPE(1:1) .EQ. 'R' ) THEN
         F = 0.0
         G = 1.0
      ENDIF

      YV( 1 ) = F
      YV( 2 ) = G
      YV( 3 ) = 0.0
      YV( 4 ) = 0.0
      YV( 5 ) = 0.0

C     *** Acousto-elastic half-space ***

      IF ( BCTYPE(1:1) .EQ. 'A' ) THEN
         IF ( REAL( CSHS ) .GT. 0.0 ) THEN
            GAMS2 = X - OMEGA2 / DBLE( CSHS ) ** 2
            GAMP2 = X - OMEGA2 / DBLE( CPHS ) ** 2

            GAMS = SQRT( GAMS2 )
            GAMP = SQRT( GAMP2 )

            RMU = RHOHS * DBLE( CSHS ) ** 2

            YV( 1 ) = ( GAMS*GAMP - X ) / RMU
            YV( 2 ) = ( ( GAMS2 + X ) ** 2 - 4.0*GAMS * GAMP * X ) * RMU
            YV( 3 ) = 2.0 * GAMS * GAMP - GAMS2 - X
            YV( 4 ) = GAMP * ( X - GAMS2 )
            YV( 5 ) = GAMS * ( GAMS2 - X )

            F = OMEGA2 * YV( 4 )
            G = YV( 2 )
            IF ( G .GT. 0.0 ) MODECT = MODECT + 1
         ELSE
            GAMP = SQRT( X - DBLE( OMEGA2 / CPHS ** 2 ) )
            F = 1.0
            G = RHOHS / GAMP
         ENDIF
      ENDIF

      IF ( BOTTOP(1:3) .EQ. 'TOP' ) G = -G

C     *** Shoot through elastic layers ***

      IF ( BOTTOP(1:3) .EQ. 'TOP' ) THEN

C        ------ Shoot down from top
         IF ( NFACT. GT. 1 ) THEN

            DO MED = 1, NFACT - 1
               CALL ELASDN( X, YV, IPOW, MED )
            END DO

            F = OMEGA2 * YV( 4 )
            G = YV( 2 )
         ENDIF
      ELSE

C        ------ Shoot up from bottom
         IF ( NLACT. LT. NMEDIA ) THEN

            DO MED = NMEDIA, NLACT + 1, -1
               CALL ELASUP( X, YV, IPOW, MED )
            END DO

            F = OMEGA2 * YV( 4 )
            G = YV( 2 )
         ENDIF
      ENDIF

      RETURN
      END
C**********************************************************************C
      SUBROUTINE ELASUP( X, YV, IPOW, MED )

C     Propagates through an elastic layer using 
C     compound matrix formulation

      INCLUDE 'common.f'

      REAL*8 XV( 5 ), YV( 5 ), ZV( 5 )
      PARAMETER ( ROOF = 1.0E5, FLOOR = 1.0E-5, IPOWR = 5, IPOWF = -5 )

C     Euler's method for first step

      TWOX   = 2.0 * X
      TWOH   = 2.0 * H( MED )
      FOURHX = 4.0 * H( MED ) * X
      J = LOC( MED ) + N( MED ) + 1
      XB3 = X * B3( J ) - RHO( J )

      ZV(1) = YV(1) - 0.5*(   B1( J ) * YV( 4 ) - B2( J ) * YV( 5 ) )
      ZV(2) = YV(2) - 0.5*( -RHO( J ) * YV( 4 ) -     XB3 * YV( 5 ) )
      ZV(3) = YV(3) - 0.5*(      TWOH * YV( 4 ) + B4( J ) * YV( 5 ) )
      ZV(4) = YV(4) - 0.5*(   XB3*YV(1) + B2(J)*YV(2) -TWOX*B4(J)*YV(3))
      ZV(5) = YV(5) - 0.5*(RHO(J)*YV(1) - B1(J)*YV(2) -    FOURHX*YV(3))

C     Modified midpoint method

      DO I = N( MED ), 1, -1
         J = J - 1

         DO L = 1, 5
            XV( L ) = YV( L )
            YV( L ) = ZV( L )
         END DO

         XB3 = X * B3( J ) - RHO( J )

         ZV(1) = XV(1) - (   B1( J ) * YV( 4 ) - B2( J ) * YV( 5 ) )
         ZV(2) = XV(2) - ( -RHO( J ) * YV( 4 ) -     XB3 * YV( 5 ) )
         ZV(3) = XV(3) - (      TWOH * YV( 4 ) + B4( J ) * YV( 5 ) )
         ZV(4) = XV(4) - (   XB3*YV(1) + B2(J)*YV(2) - TWOX*B4(J)*YV(3))
         ZV(5) = XV(5) - (RHO(J)*YV(1) - B1(J)*YV(2) -     FOURHX*YV(3))

C         P1 = YV(2) * ( YV(4) - YV(5) ) 
C         P0 = XV(2) * ( XV(4) - XV(5) ) 
C         IF ( ( P0 .GT. 0.0 ) .AND. (P1 .LT. 0.0) ) MODECT = MODECT+1
C         IF ( ( P0 .LT. 0.0 ) .AND. (P1 .GT. 0.0) ) MODECT = MODECT+1

C        *** Scale if necessary ***

         IF ( I .NE. 1 ) THEN
            IF ( ABS( ZV( 2 ) ) .LT. FLOOR ) THEN
               DO L = 1, 5
                  ZV( L ) = ROOF * ZV( L )
                  YV( L ) = ROOF * YV( L )
               END DO
               IPOW = IPOW - IPOWR
            ENDIF

            IF ( ABS( ZV( 2 ) ) .GE. ROOF ) THEN
               DO L = 1, 5
                  ZV( L ) = FLOOR * ZV( L )
                  YV( L ) = FLOOR * YV( L )
               END DO
               IPOW = IPOW - IPOWF
            ENDIF
         ENDIF
      END DO

C     Apply the standard filter at the terminal point

      DO J = 1, 5
         YV( J ) = ( XV( J ) + 2.0 * YV( J ) + ZV( J ) ) / 4.0
      END DO

      RETURN
      END
C**********************************************************************C
      SUBROUTINE ELASDN( X, YV, IPOW, MED )

C     Propagates through an elastic layer using 
C     compound matrix formulation

      INCLUDE 'common.f'

      REAL*8 XV( 5 ), YV( 5 ), ZV( 5 )
      PARAMETER ( ROOF = 1.0E5, FLOOR = 1.0E-5, IPOWR = 5, IPOWF = -5 )

C     Euler's method for first step

      TWOX   = 2.0 * X
      TWOH   = 2.0 * H( MED )
      FOURHX = 4.0 * H( MED ) * X
      J = LOC( MED ) + 1
      XB3 = X * B3( J ) - RHO( J )

      ZV(1) = YV(1) + 0.5*(   B1( J ) * YV( 4 ) - B2( J ) * YV( 5 ) )
      ZV(2) = YV(2) + 0.5*( -RHO( J ) * YV( 4 ) -     XB3 * YV( 5 ) )
      ZV(3) = YV(3) + 0.5*(      TWOH * YV( 4 ) + B4( J ) * YV( 5 ) )
      ZV(4) = YV(4) + 0.5*(   XB3*YV(1) + B2(J)*YV(2) -TWOX*B4(J)*YV(3))
      ZV(5) = YV(5) + 0.5*(RHO(J)*YV(1) - B1(J)*YV(2) -    FOURHX*YV(3))

C     Modified midpoint method

      DO I = 1, N( MED )
         J = J + 1

         DO L = 1, 5
            XV( L ) = YV( L )
            YV( L ) = ZV( L )
         END DO

         XB3 = X * B3( J ) - RHO( J )

         ZV(1) = XV(1) + (   B1( J ) * YV( 4 ) - B2( J ) * YV( 5 ) )
         ZV(2) = XV(2) + ( -RHO( J ) * YV( 4 ) -     XB3 * YV( 5 ) )
         ZV(3) = XV(3) + (      TWOH * YV( 4 ) + B4( J ) * YV( 5 ) )
         ZV(4) = XV(4) + (   XB3*YV(1) + B2(J)*YV(2) - TWOX*B4(J)*YV(3))
         ZV(5) = XV(5) + (RHO(J)*YV(1) - B1(J)*YV(2) -     FOURHX*YV(3))

C        *** Scale if necessary ***

         IF ( I .NE. N( MED ) ) THEN
            IF ( ABS( ZV( 2 ) ) .LT. FLOOR ) THEN

               DO L = 1, 5
                  ZV( L ) = ROOF * ZV( L )
                  YV( L ) = ROOF * YV( L )
               END DO

               IPOW = IPOW - IPOWR
            ENDIF

            IF ( ABS( ZV( 2 ) ) .GE. ROOF ) THEN

               DO L = 1, 5
                  ZV( L ) = FLOOR * ZV( L )
                  YV( L ) = FLOOR * YV( L )
               END DO

               IPOW = IPOW - IPOWF
            ENDIF
         ENDIF
      END DO

C     Apply the standard filter at the terminal point

      DO J = 1, 5
         YV( J ) = ( XV( J ) + 2.0 * YV( J ) + ZV( J ) ) / 4.0
      END DO

      RETURN
      END
