      PROGRAM CAPON

C     Takes covariance matrix and replica shade file and computes a Capon
C     (MLM) surface

      INTEGER REPFIL, COVFIL, AMSFIL
      PARAMETER ( REPFIL = 20, COVFIL = 21,
     &   AMSFIL = 25, MAXNSD = 4094, MAXNRD = 4094, MAXNR = 4094,
     &   MXPHONES = 50, TOL = 0.0001, EPS = 1.0E-5 )

      INTEGER IRDREP( MAXNRD )
      REAL    R( MAXNR ), SD( MAXNSD ), RD( MAXNRD ), SURF( 1 ),
     &        RDREP( MAXNRD )
      COMPLEX P( MXPHONES, MAXNR ), PC( MAXNR )
      COMPLEX COV( MXPHONES, MXPHONES ), Y( MXPHONES ), T
      CHARACTER TITLE*80, TITLEREP*80, PLTTYP*10

      POWERMAX = 0.0
      ISDMAX   = 0
      IRMAX    = 0

C     *** Read covariance matrix ***

      OPEN ( FILE = 'COVFIL', UNIT = COVFIL, STATUS = 'OLD' )

      READ( COVFIL, * ) TITLE
      READ( COVFIL, * ) FREQ
      READ( COVFIL, * ) NRD

      IF ( NRD .GT. MXPHONES ) STOP "NRD > MXPHONES"

      DO IRD = 1, NRD
         READ( COVFIL, * ) RD( IRD )
      END DO

      DO I = 1, NRD
         DO J = 1, NRD
            READ( COVFIL, * ) I1, J1, COV( I, J )
         END DO
      END DO

C     *** Diagonal loading for stability of inversion ***

      DO I = 1, NRD
         COV( I, I ) = COV( I, I ) + EPS
      END DO

C     *** Factor the covariance matrix into L L* form

      CALL CHOLESKI( COV, MXPHONES, NRD )

C     *** Form power field (SURF = 1.0) ***

C     *** Read replica file header ***

      CALL RDHEAD( REPFIL, 'REPFIL', MAXNR, MAXNSD, MAXNRD,
     &   FREQREP, R, NR, DELTAR, PLTTYP, XS, YS, THETA,
     &   SD, NSD, RDREP, NRDREP, TITLEREP )

      IF ( FREQ .NE. FREQREP ) THEN
         WRITE( *, * ) 'Mismatch in frequencies'
         STOP
      ENDIF

C     --- Find pointers to receiver depths in replica file

      DO IRD = 1, NRD
         IRDREP( IRD ) = NEIGH( RD( IRD ), RDREP, NRDREP )
         IF ( ABS( RDREP( IRDREP( IRD ) ) - RD( IRD ) ) .GT. TOL )
     &      STOP "No matching depth in replica file"
      END DO

C     *** Form header for power field (SURF = 1.0)

      SURF( 1 )  = 1.0
      NSURF = 1

      CALL HEADER( TITLE, SURF, NSURF, SD, NSD,
     &   R, NR, FREQ, ATTEN, PLTTYP, XS, YS, THETA )

      IRECOUT = 6

C     *** Loop over trial source depths ***

      DO ISD = 1, NSD

         DO IRD = 1, NRD
C           --- read in replica vector
            IRECIN = 6 + NRD * ( ISD - 1 ) + IRDREP( IRD )
            READ ( REPFIL, REC = IRECIN ) ( P( IRD, IR ), IR = 1, NR )
         END DO

C        *** Loop over trial source ranges *** 
         DO IR = 1, NR

C           --- compute power of replica vector
            SUMREP = 0.0
            DO I = 1, NRD
               SUMREP = SUMREP + P( I, IR ) * CONJG( P( I, IR ) )
            END DO

C           *** form Pc = e* R^(-1) e taking advantage of symmetry ***

C           --- start with y = L^(-1) P
            Y( 1 ) = P( 1, IR ) / COV( 1, 1 )
            DO I = 2, NRD
               T = 0.0
               DO J = 1, I-1
                  T = T + CONJG( COV( J, I ) ) * Y( J )
               END DO
               Y( I ) = ( P( I, IR ) - T ) / COV( I, I )
            END DO

C           --- compute y*y
            POWER  = 0.0
            DO I = 1, NRD
               POWER = POWER + Y( I ) * CONJG( Y( I ) )
            END DO

            POWER = SUMREP / POWER  ! Capon is (x*x) / (x*Rx)

C           --- make note of peak location
            IF ( POWER .GT. POWERMAX ) THEN
               POWERMAX = POWER
               ISDMAX   = ISD
               IRMAX    = IR
            ENDIF

C           --- Normalize
            IF( POWER .NE. 0.0 ) THEN
C              --- take sqrt to go from power back to amplitude
C              --- take reciprocal to get proper sign in plot
               PC( IR ) = 1.0 / SQRT( POWER )
            ELSE
               PC( IR ) = 0.0
            ENDIF
         END DO

C        --- Write out Capon power vector
         IRECOUT = IRECOUT + 1
         WRITE( AMSFIL, REC = IRECOUT ) ( PC( IR ), IR = 1, NR )

      END DO        ! next source depth

      WRITE( *, * ) 'Peak power   = ', POWERMAX
      WRITE( *, * ) 'Source depth = ', SD( ISDMAX )
      WRITE( *, * ) 'Source range = ', R(   IRMAX )

      STOP
      END
